<?php

class KewegoApiException extends exception {}

class KewegoApi {

  protected $persistent = array();
  
  public function __construct($appKey, $options = array())
  {
    if (!$appKey) {
      throw new KewegoApiException('No Application Key');
    }
    $this->appKey = $appKey;
    if (!extension_loaded('curl')) {
      throw new KewegoApiException('cURL not loaded');
    }
    foreach ($options as $name => $value) {
      $this->setOption($name, $value);
    }
  }
  
  protected $options = array(
    'api_url' => 'http://api.kewego.com',
  );
  
  public function setOption($name, $value) {
    $this->options[$name] = $value;
  }
  
  public function getOption($name) {
    if (isset($this->options[$name])) return $this->options[$name];
    return null;
  }
  
  protected function prepareUrl($url, $params, $get = true) {
    $parsed_url = parse_url($url);
    if (isset($parsed_url['query'])) {
      $parsed_url['query'] = parse_str($parsed_url['query']);
    }
    else {
      $parsed_url['query'] = array();
    }
    
    $url = (object) array();
    
    $url->params = array_merge($parsed_url['query'], $params);
    $parsed_url['query'] = http_build_query($url->params, '', '&');
    
    if (!$get) {
      unset($parsed_url['query']);
    }
    $url->url = self::unparse_url($parsed_url);
    return $url;
  }
  
  public function apiCall($function, $params = array()) {
    $this->initApi();
    $params += $this->persistent;
    return $this->_apiCall($function, $params);
  }
  
  protected function initApi() {
    if (isset($this->persistent['appToken'])) return;
    $res = $this->_apiCall('app/getToken', array('appKey' => $this->appKey));
    if ($res && isset($res->appToken)) {
      $this->persistent['appToken'] = (string) $res->appToken;
    }
    else {
      throw new KewegoApiException('Cannot init Kewego API');
    }
  }
  
  protected function _apiCall($function, $params = array()) {

    $url = $this->getOption('api_url') . '/' . $function;
    
    $url = $this->prepareUrl($url, $params);
  
    $ch = curl_init($url->url);

    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_POST, 0);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);

    if ($proxy = $this->getOption('proxy')) {
      curl_setopt($ch, CURLOPT_PROXY, $proxy);
      if ($userpwd = $this->getOption('proxyuserpwd')) {
        curl_setopt($ch, CURLOPT_PROXYUSERPWD, $userpwd);
      }
    }

    $res = curl_exec($ch);
    if ($res === FALSE) {
      $err = curl_error($ch);
      curl_close($ch);
      throw new KewegoApiException("cURL error : $err");
    }
    
    curl_close($ch);
    
    $xml = simplexml_load_string($res);
    if ($xml->message) {
      return $xml->message;
    }
    throw new KewegoApiException("Kewego API error : $function");
  }

  static function data_encode(array $data, $keyprefix = "", $keypostfix = "") {
    assert( is_array($data) );
    $vars=null;
    foreach($data as $key=>$value) {
      if(is_array($value)) $vars .= data_encode($value, $keyprefix.$key.$keypostfix.urlencode("["), urlencode("]"));
      else $vars .= $keyprefix.$key.$keypostfix."=".urlencode($value)."&";
    }
    return $vars;
  }  
  
  static public function unparse_url($parsed_url) {
    $scheme   = isset($parsed_url['scheme']) ? $parsed_url['scheme'] . '://' : '';
    $host     = isset($parsed_url['host']) ? $parsed_url['host'] : '';
    $port     = isset($parsed_url['port']) ? ':' . $parsed_url['port'] : '';
    $user     = isset($parsed_url['user']) ? $parsed_url['user'] : '';
    $pass     = isset($parsed_url['pass']) ? ':' . $parsed_url['pass']  : '';
    $pass     = ($user || $pass) ? "$pass@" : '';
    $path     = isset($parsed_url['path']) ? $parsed_url['path'] : '';
    $query    = isset($parsed_url['query']) ? '?' . $parsed_url['query'] : '';
    $fragment = isset($parsed_url['fragment']) ? '#' . $parsed_url['fragment'] : '';
    return "$scheme$user$pass$host$port$path$query$fragment";
  }
}